package envoy.client.ui.renderer;

import java.awt.*;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.EnumMap;

import javax.swing.*;

import envoy.client.data.Settings;
import envoy.client.ui.Color;
import envoy.client.ui.IconUtil;
import envoy.client.ui.Theme;
import envoy.client.ui.list.ComponentList;
import envoy.client.ui.list.ComponentListCellRenderer;
import envoy.data.Message;
import envoy.data.Message.MessageStatus;

/**
 * Defines how a message is displayed.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>MessageListRenderer.java</strong><br>
 * Created: <strong>19 Oct 2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @author Leon Hofmeister
 * @since Envoy v0.1-alpha
 */
public class MessageListRenderer implements ComponentListCellRenderer<Message> {

	private static EnumMap<MessageStatus, ImageIcon> statusIcons;

	static {
		try {
			statusIcons = IconUtil.loadByEnum(MessageStatus.class, 16);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	@Override
	public JPanel getListCellComponent(ComponentList<? extends Message> list, Message message, boolean isSelected) {
		final Theme theme = Settings.getInstance().getThemes().get(Settings.getInstance().getCurrentTheme());

		// panel
		final JPanel panel = new JPanel();

		GridBagLayout gbl_panel = new GridBagLayout();
		gbl_panel.columnWidths	= new int[] { 1, 1 };
		gbl_panel.rowHeights	= new int[] { 1, 1 };
		gbl_panel.columnWeights	= new double[] { 1, 1 };
		gbl_panel.rowWeights	= new double[] { 1, 1 };

		panel.setLayout(gbl_panel);
		panel.setBackground(isSelected ? theme.getSelectionColor() : theme.getCellColor());

		// TODO: Handle message attachments

		// content variables
		final String	status	= message.getStatus().toString();
		final String	date	= new SimpleDateFormat("dd.MM.yyyy HH:mm").format(message.getCreationDate());
		final String	text	= message.getText();

		// date Label - The Label that displays the creation date of a message
		JLabel dateLabel = new JLabel();
		dateLabel.setText(date);
		dateLabel.setForeground(theme.getDateColor());
		dateLabel.setAlignmentX(1f);
		dateLabel.setFont(new Font("Arial", Font.PLAIN, 12));
		dateLabel.setPreferredSize(dateLabel.getPreferredSize());

		GridBagConstraints gbc_dateLabel = new GridBagConstraints();
		gbc_dateLabel.fill	= GridBagConstraints.BOTH;
		gbc_dateLabel.gridx	= 0;
		gbc_dateLabel.gridy	= 0;
		panel.add(dateLabel, gbc_dateLabel);


		// Message area - The JTextArea that displays the text content of a message.
		var messageTextArea = new JTextArea(text);
		messageTextArea.setLineWrap(true);
		messageTextArea.setWrapStyleWord(true);
		messageTextArea.setForeground(theme.getMessageTextColor());
		messageTextArea.setAlignmentX(0.5f);
		messageTextArea.setBackground(Color.red);
		messageTextArea.setEditable(false);
		messageTextArea.setFont(new Font("Arial", Font.PLAIN, 14));
		// messageTextArea.setPreferredSize(messageTextArea.getPreferredSize());

		GridBagConstraints gbc_messageTextArea = new GridBagConstraints();
		gbc_messageTextArea.fill	= GridBagConstraints.HORIZONTAL;
		gbc_messageTextArea.gridx	= 0;
		gbc_messageTextArea.gridy	= 1;
		panel.add(messageTextArea, gbc_messageTextArea);


		// status Label - displays the status of the message
		JLabel statusLabel = new JLabel(statusIcons.get(message.getStatus()));

		GridBagConstraints gbc_statusLabel = new GridBagConstraints();
		gbc_statusLabel.fill	= GridBagConstraints.BOTH;
		gbc_statusLabel.gridx	= 1;
		gbc_statusLabel.gridy	= 1;
		panel.add(statusLabel, gbc_statusLabel);

		// Forwarding
		if (message.isForwarded()) try {
			var forwardLabel = new JLabel("Forwarded", new ImageIcon(ClassLoader.getSystemResourceAsStream(null).readAllBytes()),
					SwingConstants.CENTER);
			forwardLabel.setBackground(panel.getBackground());
			forwardLabel.setForeground(Color.lightGray);
			
			GridBagConstraints gbc_forwardLabel = new GridBagConstraints();
			gbc_forwardLabel.fill	= GridBagConstraints.BOTH;
			gbc_forwardLabel.gridx	= 1;
			gbc_forwardLabel.gridy	= 0;
			panel.add(forwardLabel, gbc_forwardLabel);
		} catch (IOException e) {
			e.printStackTrace();
		}

		int padding = (int) (list.getWidth() * 0.35);
		// Define some space to the messages below
		panel.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createEmptyBorder(0, 0, 0, padding),
				BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(), BorderFactory.createEtchedBorder())));

		var size = new Dimension(list.getWidth(), (int) panel.getPreferredSize().getHeight());

		// panel.setPreferredSize(panel.getPreferredSize());
		panel.setMaximumSize(size);
		// System.out.println(panel.getMaximumSize());
		return panel;
	}
}
